import unittest

import numpy as np
import tensorflow as tf
from transformers_keras.modeling_bert import *


class ModelingBertTest(tf.test.TestCase):

    def testBertEmbeddings(self):
        embedding_layer = BertEmbedding(vocab_size=16, max_positions=40)
        inputs = (
            tf.constant([[0, 2, 3, 4, 5, 1]]),
            # tf.constant([[0, 1, 2, 3, 4, 5]]),
            tf.constant([[0, 0, 0, 1, 1, 1]]),
        )
        embeddings = embedding_layer(inputs, mode='embedding', training=True)
        self.assertAllEqual([1, 6, 768], embeddings.shape)

        inputs2 = tf.random.uniform(shape=(1, 768))
        outputs = embedding_layer(inputs2, mode='linear')
        self.assertAllEqual([1, 16], outputs.shape)

    def testBertIntermediate(self):
        inter = BertIntermediate()
        inputs = tf.random.uniform(shape=(2, 16, 768))
        outputs = inter(inputs)
        self.assertAllEqual([2, 16, 3072], outputs.shape)

    def testBertEncoderLayer(self):
        encoder = BertEncoderLayer()
        hidden_states = tf.random.uniform((2, 10, 768))
        attention_mask = None
        outputs, attention_weights = encoder(
            inputs=(hidden_states, attention_mask), training=True)

        self.assertAllEqual([2, 10, 768], outputs.shape)
        self.assertAllEqual([2, 8, 10, 10], attention_weights.shape)

    def testBertEncoder(self):
        encoder = BertEncoder(num_layers=2)

        hidden_states = tf.random.uniform((2, 10, 768))
        attention_mask = None
        outputs, all_hidden_states, all_attention_scores = encoder(
            inputs=(hidden_states, attention_mask), training=True)

        self.assertAllEqual([2, 10, 768], outputs.shape)

        self.assertAllEqual(2, len(all_hidden_states))
        for state in all_hidden_states:
            self.assertAllEqual([2, 10, 768], state.shape)

        self.assertAllEqual(2, len(all_attention_scores))
        for attention in all_attention_scores:
            self.assertAllEqual([2, 8, 10, 10], attention.shape)

    def testBertPooler(self):
        pooler = BertPooler()
        inputs = tf.random.uniform(shape=(2, 16, 768))
        outputs = pooler(inputs)
        self.assertAllEqual([2, 768], outputs.shape)

    def _build_bert_inputs(self):
        input_ids = tf.constant(
            [1, 2, 3, 4, 5, 6, 7, 5, 3, 2, 3, 4, 1, 2, 3, 1, 2, 3, 4, 5, 6, 6, 6, 7, 7, 8, 0, 0, 0, 0, 0, 0],
            shape=(2, 16),
            dtype=np.int32)  # input_ids
        token_type_ids = np.array(
            [[0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1]], dtype=np.int64)  # token_type_ids,
        input_mask = tf.constant(
            [[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0],
             [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0]], dtype=np.float32)  # input_mask
        return input_ids, token_type_ids, input_mask

    def _check_bert_outputs(self, return_states=False, return_attention_weights=False):
        model = Bert(
            vocab_size=100,
            num_layers=2,
            return_states=return_states,
            return_attention_weights=return_attention_weights)
        outputs = model(inputs=self._build_bert_inputs())
        sequence_outputs, pooled_outputs = outputs[0], outputs[1]
        self.assertAllEqual([2, 16, 768], sequence_outputs.shape)
        self.assertAllEqual([2, 768], pooled_outputs.shape)

        all_states, all_attn_weights = None, None
        if return_states and return_attention_weights:
            self.assertEqual(4, len(outputs))
            all_states, all_attn_weights = outputs[2], outputs[3]
        elif return_states and not return_attention_weights:
            self.assertEqual(3, len(outputs))
            all_states = outputs[2]
        elif not return_states and return_attention_weights:
            self.assertEqual(3, len(outputs))
            all_attn_weights = outputs[2]
        else:
            self.assertEqual(2, len(outputs))

        if all_states is not None:
            self.assertEqual(2, len(all_states))
            for state in all_states:
                self.assertAllEqual([2, 16, 768], state.shape)

        if all_attn_weights is not None:
            self.assertEqual(2, len(all_attn_weights))
            for attention in all_attn_weights:
                self.assertAllEqual([2, 8, 16, 16], attention.shape)

    def testBert(self):
        self._check_bert_outputs(return_states=True, return_attention_weights=True)
        self._check_bert_outputs(return_states=True, return_attention_weights=False)
        self._check_bert_outputs(return_states=False, return_attention_weights=True)
        self._check_bert_outputs(return_states=False, return_attention_weights=False)

    def testBertMLMHead(self):
        embedding = BertEmbedding(vocab_size=100)
        mlm = BertMLMHead(vocab_size=100, embedding=embedding)

        inputs = tf.random.uniform(shape=(2, 16, 768))
        outputs = mlm(inputs)
        self.assertAllEqual([2, 16, mlm.vocab_size], outputs.shape)

    def testBertNSPHead(self):
        nsp = BertNSPHead()
        inputs = tf.random.uniform(shape=(2, 768))
        outputs = nsp(inputs)
        self.assertAllEqual([2, 2], outputs.shape)


if __name__ == "__main__":
    unittest.main()



import json
import logging
import os

import tensorflow as tf

from .abstract_adapter import AbstractAdapter, zip_weights


class BertAdapter(AbstractAdapter):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def adapte_config(self, config_file, **kwrgs):
        with open(config_file, mode='rt', encoding='utf8') as fin:
            config = json.load(fin)

        model_config = {
            'vocab_size': config['vocab_size'],
            'activation': config['hidden_act'],
            'max_positions': config['max_position_embeddings'],
            'hidden_size': config['hidden_size'],
            'type_vocab_size': config['type_vocab_size'],
            'intermediate_size': config['intermediate_size'],
            'hidden_dropout_rate': config['hidden_dropout_prob'],
            'attention_dropout_rate': config['attention_probs_dropout_prob'],
            'stddev': config['initializer_range'],
            'num_layers': config['num_hidden_layers'],
            'num_attention_heads': config['num_attention_heads'],
        }
        return model_config

    def adapte_weights(self, model, config, ckpt, **kwargs):
        # mapping weight names
        weights_mapping = self._mapping_weight_names(config['num_layers'])
        # zip weight names and values
        zipped_weights = zip_weights(
            model,
            ckpt,
            weights_mapping,
            verbose=kwargs.get('verbose', True))
        # set values to weights
        tf.keras.backend.batch_set_value(zipped_weights)

    def _mapping_weight_names(self, num_layers=12):
        mapping = {}

        if not self.skip_token_embedding:
            mapping['bert/embedding/weight:0'] = 'bert/embeddings/word_embeddings'
        if not self.skip_position_embedding:
            mapping['bert/embedding/position_embedding/embeddings:0'] = 'bert/embeddings/position_embeddings'
        if not self.skip_segment_embedding:
            mapping['bert/embedding/token_type_embedding/embeddings:0'] = 'bert/embeddings/token_type_embeddings'
        if not self.skip_embedding_layernorm:
            mapping['bert/embedding/layer_norm/gamma:0'] = 'bert/embeddings/LayerNorm/gamma'
            mapping['bert/embedding/layer_norm/beta:0'] = 'bert/embeddings/LayerNorm/beta'

        # encoder
        model_prefix = 'bert/encoder/layer_{}'
        for i in range(num_layers):
            encoder_prefix = 'bert/encoder/layer_{}/'.format(i)
            # attention
            attention_prefix = encoder_prefix + 'attention/'
            for n in ['query', 'key', 'value']:
                for w in ['kernel', 'bias']:
                    mapping[attention_prefix + n + '/' + w + ':0'] = attention_prefix + 'self/' + n + '/' + w
            #
            mapping[attention_prefix + 'dense/kernel:0'] = attention_prefix + 'output/dense/kernel'
            mapping[attention_prefix + 'dense/bias:0'] = attention_prefix + 'output/dense/bias'
            mapping[attention_prefix + 'layer_norm/gamma:0'] = attention_prefix + 'output/LayerNorm/gamma'
            mapping[attention_prefix + 'layer_norm/beta:0'] = attention_prefix + 'output/LayerNorm/beta'
            # intermediate
            intermediate_prefix = encoder_prefix + 'intermediate/'
            mapping[intermediate_prefix + 'dense/kernel:0'] = intermediate_prefix + 'dense/kernel'
            mapping[intermediate_prefix + 'dense/bias:0'] = intermediate_prefix + 'dense/bias'
            # output
            mapping[encoder_prefix + 'dense/kernel:0'] = encoder_prefix + 'output/dense/kernel'
            mapping[encoder_prefix + 'dense/bias:0'] = encoder_prefix + 'output/dense/bias'
            mapping[encoder_prefix + 'layer_norm/gamma:0'] = encoder_prefix + 'output/LayerNorm/gamma'
            mapping[encoder_prefix + 'layer_norm/beta:0'] = encoder_prefix + 'output/LayerNorm/beta'

        # pooler
        if not self.skip_pooler:
            mapping['bert/pooler/dense/kernel:0'] = 'bert/pooler/dense/kernel'
            mapping['bert/pooler/dense/bias:0'] = 'bert/pooler/dense/bias'

        return mapping



import json
import logging
import os

import tensorflow as tf

from transformers_keras.adapters import parse_pretrained_model_files
from transformers_keras.adapters.bert_adapter import BertAdapter

from .layers import MultiHeadAttention
from .modeling_utils import (choose_activation, initialize, unpack_inputs_2,
                             unpack_inputs_3)


class BertEmbedding(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=1,
                 max_positions=512,
                 embedding_size=768,
                 type_vocab_size=2,
                 dropout_rate=0.2,
                 stddev=0.02,
                 epsilon=1e-12,
                 **kwargs):
        super().__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.embedding_size = embedding_size
        self.stddev = stddev

        self.position_embedding = tf.keras.layers.Embedding(
            max_positions,
            embedding_size,
            embeddings_initializer=initialize(stddev),
            name='position_embedding'
        )
        self.token_type_embedding = tf.keras.layers.Embedding(
            type_vocab_size,
            embedding_size,
            embeddings_initializer=initialize(stddev),
            name='token_type_embedding'
        )
        self.layer_norm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='layer_norm')
        self.dropout = tf.keras.layers.Dropout(dropout_rate)

    def build(self, input_shape):
        self.token_embedding = self.add_weight(
            'weight',
            shape=[self.vocab_size, self.embedding_size],
            initializer=initialize(self.stddev)
        )
        super().build(input_shape)

    def call(self, inputs, mode='embedding', training=None):
        # used for masked lm
        if mode == 'linear':
            return tf.matmul(inputs, self.token_embedding, transpose_b=True)

        input_ids, token_type_ids = unpack_inputs_2(inputs)
        seq_len = tf.shape(input_ids)[1]
        position_ids = tf.range(seq_len, dtype=tf.int32)[tf.newaxis, :]

        position_embeddings = self.position_embedding(position_ids)
        token_type_embeddings = self.token_type_embedding(token_type_ids)
        token_embeddings = tf.gather(self.token_embedding, input_ids)

        embeddings = token_embeddings + token_type_embeddings + position_embeddings
        embeddings = self.layer_norm(embeddings)
        embeddings = self.dropout(embeddings, training=training)
        return embeddings


class BertIntermediate(tf.keras.layers.Layer):

    def __init__(self, intermediate_size=3072, activation='gelu', stddev=0.02, **kwargs):
        super().__init__(**kwargs)
        self.dense = tf.keras.layers.Dense(
            intermediate_size, kernel_initializer=initialize(stddev), name='dense')
        self.activation = choose_activation(activation)

    def call(self, inputs, training=None):
        hidden_states = inputs
        hidden_states = self.dense(hidden_states)
        hidden_states = self.activation(hidden_states)
        return hidden_states


class BertEncoderLayer(tf.keras.layers.Layer):

    def __init__(self,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 epsilon=1e-12,
                 stddev=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        # attention block
        self.attention = MultiHeadAttention(
            hidden_size=hidden_size,
            num_attention_heads=num_attention_heads,
            hidden_dropout_rate=hidden_dropout_rate,
            attention_dropout_rate=attention_dropout_rate,
            name='attention')
        # intermediate block
        self.intermediate = BertIntermediate(
            intermediate_size=intermediate_size,
            activation=activation,
            stddev=stddev,
            name='intermediate')
        # output block
        self.output_dense = tf.keras.layers.Dense(hidden_size, kernel_initializer=initialize(stddev), name='dense')
        self.output_dropout = tf.keras.layers.Dropout(hidden_dropout_rate)
        self.output_layer_norm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='layer_norm')

    def call(self, inputs, training=None):
        hidden_states, mask = inputs
        attn_output, attn_weights = self.attention(inputs=(hidden_states, hidden_states, hidden_states, mask))
        outputs = self.intermediate(inputs=attn_output)
        outputs = self.output_dropout(self.output_dense(outputs), training=training)
        outputs = self.output_layer_norm(attn_output + outputs)
        return outputs, attn_weights


class BertEncoder(tf.keras.layers.Layer):

    def __init__(self,
                 num_layers=6,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 epsilon=1e-12,
                 stddev=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        self.encoder_layers = [
            BertEncoderLayer(
                hidden_size=hidden_size,
                num_attention_heads=num_attention_heads,
                intermediate_size=intermediate_size,
                activation=activation,
                hidden_dropout_rate=hidden_dropout_rate,
                attention_dropout_rate=attention_dropout_rate,
                epsilon=epsilon,
                stddev=stddev,
                name='layer_{}'.format(i)
            ) for i in range(num_layers)
        ]

    def call(self, inputs, training=None):
        hidden_states, attention_mask = inputs
        all_hidden_states = []
        all_attention_scores = []
        for _, encoder in enumerate(self.encoder_layers):
            hidden_states, attention_score = encoder(inputs=(hidden_states, attention_mask))
            all_hidden_states.append(hidden_states)
            all_attention_scores.append(attention_score)

        return hidden_states, all_hidden_states, all_attention_scores


class BertPooler(tf.keras.layers.Layer):

    def __init__(self, hidden_size=768, stddev=0.02, **kwargs):
        super().__init__(**kwargs)
        self.dense = tf.keras.layers.Dense(
            hidden_size, kernel_initializer=initialize(stddev), activation='tanh', name='dense')

    def call(self, inputs, training=None):
        hidden_states = inputs
        # pool the first token: [CLS]
        outputs = self.dense(hidden_states[:, 0])
        return outputs


class Bert(tf.keras.Model):

    def __init__(self,
                 vocab_size=1,
                 max_positions=512,
                 hidden_size=768,
                 type_vocab_size=2,
                 num_layers=6,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 stddev=0.02,
                 epsilon=1e-12,
                 return_states=False,
                 return_attention_weights=False,
                 **kwargs):
        kwargs.pop('name', None)
        super().__init__(name='bert', **kwargs)
        self.bert_embedding = BertEmbedding(
            vocab_size=vocab_size,
            max_positions=max_positions,
            embedding_size=hidden_size,
            type_vocab_size=type_vocab_size,
            dropout_rate=hidden_dropout_rate,
            stddev=stddev,
            epsilon=epsilon,
            name='embedding')

        self.bert_encoder = BertEncoder(
            num_layers=num_layers,
            hidden_size=hidden_size,
            num_attention_heads=num_attention_heads,
            intermediate_size=intermediate_size,
            activation=activation,
            hidden_dropout_rate=hidden_dropout_rate,
            attention_dropout_rate=attention_dropout_rate,
            epsilon=epsilon,
            stddev=stddev,
            name='encoder')

        self.bert_pooler = BertPooler(hidden_size=hidden_size, stddev=stddev, name='pooler')

        self.return_states = return_states
        self.return_attention_weights = return_attention_weights

    def call(self, inputs, training=None):
        input_ids, token_type_ids, mask = unpack_inputs_3(inputs)
        mask = mask[:, tf.newaxis, tf.newaxis, :]  # (batch_size, seq_len) -> (batch_size, 1, 1, seq_len)
        embedding = self.bert_embedding(inputs=(input_ids, token_type_ids), mode='embedding')
        output, all_hidden_states, all_attention_scores = self.bert_encoder(inputs=(embedding, mask))
        pooled_output = self.bert_pooler(output)
        outputs = (output, pooled_output)
        if self.return_states:
            outputs += (all_hidden_states, )
        if self.return_attention_weights:
            outputs += (all_attention_scores, )
        return outputs

    def dummy_inputs(self):
        input_ids = tf.constant([0] * 128, dtype=tf.int64, shape=(1, 128))
        segment_ids = tf.constant([0] * 128, dtype=tf.int64, shape=(1, 128))
        #mask = tf.constant([1] * 128, dtype=tf.int64, shape=(1, self.max_positions))
        return (input_ids, segment_ids)

    @classmethod
    def from_pretrained(cls, pretrained_model_dir, adapter=None, verbose=True, **kwargs):
        config_file, ckpt, vocab_file = parse_pretrained_model_files(pretrained_model_dir)
        if not adapter:
            adapter = BertAdapter(**kwargs)
        model_config = adapter.adapte_config(config_file, **kwargs)
        model_config['return_states'] = kwargs.get('return_states', False)
        model_config['return_attention_weights'] = kwargs.get('return_attention_weights', False)
        model = cls(**model_config)
        model(model.dummy_inputs())
        adapter.adapte_weights(model, model_config, ckpt, **kwargs)
        return model


class BertMLMHead(tf.keras.layers.Layer):
    """Masked language model for BERT pre-training."""

    def __init__(self,
                 embedding,
                 vocab_size=-1,
                 hidden_size=768,
                 activation='gelu',
                 epsilon=1e-12,
                 stddev=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.embedding = embedding
        self.activation = choose_activation(activation)
        self.dense = tf.keras.layers.Dense(hidden_size, kernel_initializer=initialize(stddev), name='dense')
        self.layer_norm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='layer_norm')

    def build(self, input_shape):
        self.bias = self.add_weight(shape=(self.vocab_size,), initializer='zeros', trainable=True, name='bias')
        super().build(input_shape)

    def call(self, inputs, training=None):
        hidden_states = inputs
        hidden_states = self.dense(hidden_states)
        hidden_states = self.activation(hidden_states)
        hidden_states = self.layer_norm(hidden_states)
        hidden_states = self.embedding(inputs=hidden_states, mode='linear')
        hidden_states = hidden_states + self.bias
        return hidden_states


class BertNSPHead(tf.keras.layers.Layer):
    """Next sentence prediction for BERT pre-training."""

    def __init__(self, stddev=0.02, **kwargs):
        super().__init__(**kwargs)
        self.classifier = tf.keras.layers.Dense(2, kernel_initializer=initialize(stddev), name='dense')

    def call(self, inputs, training=None):
        pooled_output = inputs
        relation = self.classifier(pooled_output)
        return relation



import tensorflow as tf
from transformers_keras.losses import (MaskedSparseCategoricalCrossentropy,
                                       masked_sparse_categorical_crossentropy)


class LossesTest(tf.test.TestCase):

    def testMaskedSparseCategoricalCrossentropy(self):
        x = tf.constant([1, 2, 3, 4, 0, 0], shape=(2, 3), dtype=tf.int32)
        y0 = tf.constant([
            [
                [.1, .5, .2, .1, .1],  # 1
                [.1, .2, .5, .1, .1],  # 2
                [.1, .2, .1, .5, .1],  # 3
            ],
            [
                [.1, .1, .1, .2, .5],  # 4
                [.5, .1, .1, .2, .1],  # 0, masked
                [.5, .1, .1, .2, .1],  # 0, masked
            ]],
            shape=(2, 3, 5),
            dtype=tf.float32
        )
        y1 = tf.constant([
            [
                [.1, .5, .2, .1, .1],  # 1
                [.1, .2, .5, .1, .1],  # 2
                [.1, .2, .1, .5, .1],  # 3
            ],
            [
                [.1, .1, .1, .2, .5],  # 4
                [.1, .1, .1, .2, .5],  # 4, masked
                [.1, .1, .5, .2, .1],  # 2, masked
            ]],
            shape=(2, 3, 5),
            dtype=tf.float32
        )
        # the last two time steps is masking positions, so these two losses should be equal
        loss0 = masked_sparse_categorical_crossentropy(x, y0)
        loss1 = masked_sparse_categorical_crossentropy(x, y1)
        self.assertEqual(loss0, loss1)

        # the first 4 positions are the same, so masked ce and ce should be almost equal(masked ce uses epsilon=1e-6)
        ce = tf.keras.losses.SparseCategoricalCrossentropy(from_logits=False)(x, y0)
        self.assertAlmostEqual(ce.numpy(), loss0.numpy(), delta=6)

        mscc = MaskedSparseCategoricalCrossentropy(from_logits=False, axis=-1)
        _loss0 = mscc(x, y0)
        _loss1 = mscc(x, y1)
        self.assertAlmostEqual(loss0.numpy(), _loss0.numpy(), delta=6)
        self.assertAlmostEqual(loss1.numpy(), _loss1.numpy(), delta=6)


if __name__ == "__main__":
    tf.test.main()



from .tokenizer import BasicTokenizer, WordpieceTokenizer


class BertTokenizer:

    def __init__(self,
                 vocab_file,
                 pad_token='[PAD]',
                 unk_token='[UNK]',
                 cls_token='[CLS]',
                 sep_token='[SEP]',
                 mask_token='[MASK]',
                 do_lower_case=True,
                 do_basic_tokenization=True,
                 tokenize_chinese_chars=True,
                 never_split=None,
                 max_input_chars_per_word=100,
                 **kwargs):
        super().__init__()
        # load vocab dict from file
        self.vocab = self._load_vocab(vocab_file)
        self.reverse_vocab = {v: k for k, v in self.vocab.items()}
        assert len(self.vocab) == len(self.reverse_vocab)

        self.vocab_size = len(self.vocab)
        self.pad_token = pad_token
        self.unk_token = unk_token
        self.cls_token = cls_token
        self.sep_token = sep_token
        self.mask_token = mask_token
        self.pad_id = self.vocab[self.pad_token]
        self.unk_id = self.vocab[self.unk_token]
        self.cls_id = self.vocab[self.cls_token]
        self.sep_id = self.vocab[self.sep_token]
        self.mask_id = self.vocab[self.mask_token]

        self.never_split = never_split or []
        self.do_basic_tokenization = do_basic_tokenization

        if self.do_basic_tokenization:
            self.basic_tokenizer = BasicTokenizer(
                do_lower_case=do_lower_case,
                never_split=never_split,
                tokenize_chinese_chars=tokenize_chinese_chars)

        self.wordpiece_tokenizer = WordpieceTokenizer(
            vocab=set(self.vocab.keys()),
            unk_token=self.unk_token,
            max_input_chars_per_word=max_input_chars_per_word)

    def tokenize(self, text, never_split=None, **kwargs):
        if not self.do_basic_tokenization:
            return self.wordpiece_tokenizer.tokenize(text)
        tokens = []
        never_split = never_split + self.never_split if never_split is not None else self.never_split
        for token in self.basic_tokenizer.tokenize(text, never_split=never_split):
            for t in self.wordpiece_tokenizer.tokenize(token):
                tokens.append(t)
        return tokens

    def encode(self, text, add_cls=True, add_sep=True, never_split=None, **kwargs):
        ids = []
        for token in self.tokenize(text, never_split=never_split, **kwargs):
            ids.append(self.vocab.get(token, self.unk_id))
        if add_cls:
            ids = [self.cls_id] + ids
        if add_sep:
            ids = ids + [self.sep_id]
        return ids

    def decode(self, ids, drop_cls=True, drop_sep=True, **kwargs):
        tokens = [self.reverse_vocab.get(_id, self.unk_token) for _id in ids]
        if drop_cls and tokens[0] == self.cls_token:
            tokens = tokens[1:]
        if drop_sep and tokens[-1] == self.sep_token:
            tokens = tokens[:-1]
        return tokens

    def _load_vocab(self, vocab_file):
        words = []
        with open(vocab_file, mode='rt', encoding='utf-8') as fin:
            for line in fin:
                word = line.rstrip('\n')
                words.append(word)
        vocab = {}
        for idx, word in enumerate(words):
            vocab[word] = idx
        return vocab



import logging
import os
import time

import tensorflow as tf


class TransformerLearningRate(tf.keras.optimizers.schedules.LearningRateSchedule):
    """Learning rate schedule for Transformer."""

    def __init__(self, depth, warmup_steps=4000):
        """
        Args:
            depth: Python integer, the model's hidden size
            warmup_steps: Python integer, steps to warmup learning rate
        """
        self.depth = depth
        self.warmup_steps = warmup_steps

    def __call__(self, step):
        arg1 = tf.math.rsqrt(step)
        arg2 = step * (self.warmup_steps ** -1.5)
        return tf.math.rsqrt(tf.cast(self.depth, tf.float32)) * tf.math.minimum(arg1, arg2)

    def get_config(self):
        config = {
            'depth': self.depth,
            'warmup_steps': self.warmup_steps
        }
        return config


class SavedModelExporter(tf.keras.callbacks.Callback):
    """Export saved model every n epochs or every n steps."""

    def __init__(self, export_dir, every_epoch=1, every_steps=None):
        super(SavedModelExporter, self).__init__()
        self.export_dir = export_dir
        self.every_epoch = every_epoch
        self.every_steps = every_steps
        self.epoch = 0

    def on_train_batch_end(self, batch, logs=None):
        if self.every_steps is None:
            return
        if self.every_steps == 0:
            raise ValueError('`every_steps` can not be set to 0.')
        if (batch + 1) % self.every_steps == 0:
            millis = int(round(time.time() * 1000))
            p = os.path.join(self.export_dir, 'epoch-{}-step-{}-{}'.format(self.epoch, batch + 1, millis))
            if not os.path.exists(p):
                os.makedirs(p)
            self.model.save(p, include_optimizer=False, save_format='tf')
            logging.info('Export model in SavedModel format to {} at batch {}.'.format(p, batch + 1))

    def on_epoch_end(self, epoch, logs=None):
        self.epoch = epoch
        if self.every_epoch is None:
            return
        if self.every_epoch == 0:
            raise ValueError('`every_epoch` can not be set to 0.')
        if (epoch + 1) % self.every_epoch == 0:
            millis = int(round(time.time() * 1000))
            p = os.path.join(self.export_dir, 'epoch-{}-{}'.format(epoch, millis))
            if not os.path.exists(p):
                os.makedirs(p)
            self.model.save(p, include_optimizer=False, save_format='tf')
            logging.info('Export model in SavedModel format to {} at epoch {}.'.format(p, epoch))



import json
import logging
import os

import tensorflow as tf

from .abstract_adapter import AbstractAdapter, zip_weights


class AlbertAdapter(AbstractAdapter):

    def __init__(self, skip_embedding_mapping_in=False, **kwargs):
        super().__init__(**kwargs)
        self.skip_embedding_mapping_in = skip_embedding_mapping_in

    def adapte_config(self, config_file, **kwargs):
        with open(config_file, mode='rt', encoding='utf8') as fin:
            config = json.load(fin)

        model_config = {
            'vocab_size': config['vocab_size'],
            'max_positions': config['max_position_embeddings'],
            'embedding_size': config['embedding_size'],
            'type_vocab_size': config['type_vocab_size'],
            'num_layers': config['num_hidden_layers'],
            'num_groups': config['num_hidden_groups'],
            'num_layers_each_group': config['inner_group_num'],
            'hidden_size': config['hidden_size'],
            'num_attention_heads': config['num_attention_heads'],
            'intermediate_size': config['intermediate_size'],
            'activation': config['hidden_act'],
            'hidden_dropout_rate': config['hidden_dropout_prob'],
            'attention_dropout_rate': config['attention_probs_dropout_prob'],
            'stddev': config['initializer_range'],
        }
        return model_config

    def adapte_weights(self, model, config, ckpt, **kwargs):
        # mapping weight names
        weights_mapping = self._mapping_weight_names(config['num_groups'], config['num_layers_each_group'])
        # zip weights and its' values
        zipped_weights = zip_weights(
            model,
            ckpt,
            weights_mapping,
            verbose=kwargs.get('verbose', True))
        # set values to weights
        tf.keras.backend.batch_set_value(zipped_weights)

    def _mapping_weight_names(self, num_groups, num_layers_each_group):
        mapping = {}

        if not self.skip_token_embedding:
            mapping['albert/embeddings/weight:0'] = 'bert/embeddings/word_embeddings'
        if not self.skip_position_embedding:
            mapping['albert/embeddings/position_embeddings/embeddings:0'] = 'bert/embeddings/position_embeddings'
        if not self.skip_segment_embedding:
            mapping['albert/embeddings/token_type_embeddings/embeddings:0'] = 'bert/embeddings/token_type_embeddings'
        if not self.skip_embedding_layernorm:
            mapping['albert/embeddings/layer_norm/gamma:0'] = 'bert/embeddings/LayerNorm/gamma'
            mapping['albert/embeddings/layer_norm/beta:0'] = 'bert/embeddings/LayerNorm/beta'
        if not self.skip_embedding_mapping_in:
            mapping['albert/encoder/embedding_mapping/kernel:0'] = 'bert/encoder/embedding_hidden_mapping_in/kernel'
            mapping['albert/encoder/embedding_mapping/bias:0'] = 'bert/encoder/embedding_hidden_mapping_in/bias'

        # encoder
        for group in range(num_groups):
            for layer in range(num_layers_each_group):
                k_prefix = 'albert/encoder/group_{}/layer_{}/'.format(group, layer)
                v_prefix = 'bert/encoder/transformer/group_{}/inner_group_{}/'.format(group, layer)
                # attention
                for n in ['query', 'key', 'value']:
                    for x in ['kernel', 'bias']:
                        k = k_prefix + 'attention/{}/{}:0'.format(n, x)
                        v = v_prefix + 'attention_1/self/{}/{}'.format(n, x)
                        mapping[k] = v

                # attention dense
                for n in ['kernel', 'bias']:
                    k = k_prefix + 'attention/dense/{}:0'.format(n)
                    v = v_prefix + 'attention_1/output/dense/{}'.format(n)
                    mapping[k] = v

                for n in ['gamma', 'beta']:
                    # attention layer norm
                    k = k_prefix + 'attention/layer_norm/{}:0'.format(n)
                    v = v_prefix + 'LayerNorm/{}'.format(n)
                    mapping[k] = v
                    # albert encoder layer norm
                    k = k_prefix + 'layer_norm/{}:0'.format(n)
                    v = v_prefix + 'LayerNorm_1/{}'.format(n)
                    mapping[k] = v

                for n in ['kernel', 'bias']:
                    # intermediate
                    k = k_prefix + 'ffn/{}:0'.format(n)
                    v = v_prefix + 'ffn_1/intermediate/dense/{}'.format(n)
                    mapping[k] = v
                    # dense
                    k = k_prefix + 'ffn_output/{}:0'.format(n)
                    v = v_prefix + 'ffn_1/intermediate/output/dense/{}'.format(n)
                    mapping[k] = v

        # pooler
        if not self.skip_pooler:
            for n in ['kernel', 'bias']:
                k = 'albert/pooler/{}:0'.format(n)
                v = 'bert/pooler/dense/{}'.format(n)
                mapping[k] = v

        return mapping
